@@ -1,4 +1,6 @@
-## datetimelike delegation ##
+"""
+datetimelike delegation
+"""
 
 import numpy as np
 from pandas.core.base import PandasDelegate, NoNewAttributesMixin
@@ -8,13 +10,17 @@ from pandas.tseries.period import PeriodIndex
 from pandas.tseries.tdi import TimedeltaIndex
 from pandas import tslib
 from pandas.core.common import (_NS_DTYPE, _TD_DTYPE, is_period_arraylike,
-                                is_datetime_arraylike, is_integer_dtype, is_list_like,
+                                is_datetime_arraylike, is_integer_dtype,
+                                is_list_like,
                                 is_datetime64_dtype, is_datetime64tz_dtype,
                                 is_timedelta64_dtype, is_categorical_dtype,
                                 get_dtype_kinds, take_1d)
 
+
 def is_datetimelike(data):
-    """ return a boolean if we can be successfully converted to a datetimelike """
+    """
+    return a boolean if we can be successfully converted to a datetimelike
+    """
     try:
         maybe_to_datetimelike(data)
         return True
@@ -22,6 +28,7 @@ def is_datetimelike(data):
         pass
     return False
 
+
 def maybe_to_datetimelike(data, copy=False):
     """
     return a DelegatedClass of a Series that is datetimelike
@@ -42,7 +49,8 @@ def maybe_to_datetimelike(data, copy=False):
     from pandas import Series
 
     if not isinstance(data, Series):
-        raise TypeError("cannot convert an object of type {0} to a datetimelike index".format(type(data)))
+        raise TypeError("cannot convert an object of type {0} to a "
+                        "datetimelike index".format(type(data)))
 
     index = data.index
     name = data.name
@@ -51,22 +59,28 @@ def maybe_to_datetimelike(data, copy=False):
         data = orig.values.categories
 
     if is_datetime64_dtype(data.dtype):
-        return DatetimeProperties(DatetimeIndex(data, copy=copy, freq='infer'), index, name=name,
-                                                orig=orig)
+        return DatetimeProperties(DatetimeIndex(data, copy=copy, freq='infer'),
+                                  index, name=name, orig=orig)
     elif is_datetime64tz_dtype(data.dtype):
-        return DatetimeProperties(DatetimeIndex(data, copy=copy, freq='infer', ambiguous='infer'),
+        return DatetimeProperties(DatetimeIndex(data, copy=copy, freq='infer',
+                                                ambiguous='infer'),
                                   index, data.name, orig=orig)
     elif is_timedelta64_dtype(data.dtype):
-        return TimedeltaProperties(TimedeltaIndex(data, copy=copy, freq='infer'), index,
+        return TimedeltaProperties(TimedeltaIndex(data, copy=copy,
+                                                  freq='infer'), index,
                                    name=name, orig=orig)
     else:
         if is_period_arraylike(data):
-            return PeriodProperties(PeriodIndex(data, copy=copy), index, name=name, orig=orig)
+            return PeriodProperties(PeriodIndex(data, copy=copy), index,
+                                    name=name, orig=orig)
         if is_datetime_arraylike(data):
-            return DatetimeProperties(DatetimeIndex(data, copy=copy, freq='infer'), index,
+            return DatetimeProperties(DatetimeIndex(data, copy=copy,
+                                                    freq='infer'), index,
                                       name=name, orig=orig)
 
-    raise TypeError("cannot convert an object of type {0} to a datetimelike index".format(type(data)))
+    raise TypeError("cannot convert an object of type {0} to a "
+                    "datetimelike index".format(type(data)))
+
 
 class Properties(PandasDelegate, NoNewAttributesMixin):
 
@@ -80,7 +94,7 @@ class Properties(PandasDelegate, NoNewAttributesMixin):
     def _delegate_property_get(self, name):
         from pandas import Series
 
-        result = getattr(self.values,name)
+        result = getattr(self.values, name)
 
         # maybe need to upcast (ints)
         if isinstance(result, np.ndarray):
@@ -97,14 +111,16 @@ class Properties(PandasDelegate, NoNewAttributesMixin):
         result = Series(result, index=self.index, name=self.name)
 
         # setting this object will show a SettingWithCopyWarning/Error
-        result.is_copy = ("modifications to a property of a datetimelike object are not "
-                          "supported and are discarded. Change values on the original.")
+        result.is_copy = ("modifications to a property of a datetimelike "
+                          "object are not supported and are discarded. "
+                          "Change values on the original.")
 
         return result
 
     def _delegate_property_set(self, name, value, *args, **kwargs):
-        raise ValueError("modifications to a property of a datetimelike object are not "
-                         "supported. Change values on the original.")
+        raise ValueError("modifications to a property of a datetimelike "
+                         "object are not supported. Change values on the "
+                         "original.")
 
     def _delegate_method(self, name, *args, **kwargs):
         from pandas import Series
@@ -118,8 +134,9 @@ class Properties(PandasDelegate, NoNewAttributesMixin):
         result = Series(result, index=self.index, name=self.name)
 
         # setting this object will show a SettingWithCopyWarning/Error
-        result.is_copy = ("modifications to a method of a datetimelike object are not "
-                          "supported and are discarded. Change values on the original.")
+        result.is_copy = ("modifications to a method of a datetimelike object "
+                          "are not supported and are discarded. Change "
+                          "values on the original.")
 
         return result
 
@@ -141,12 +158,16 @@ class DatetimeProperties(Properties):
     def to_pydatetime(self):
         return self.values.to_pydatetime()
 
-DatetimeProperties._add_delegate_accessors(delegate=DatetimeIndex,
-                                           accessors=DatetimeIndex._datetimelike_ops,
-                                           typ='property')
-DatetimeProperties._add_delegate_accessors(delegate=DatetimeIndex,
-                                           accessors=["to_period","tz_localize","tz_convert","normalize","strftime"],
-                                           typ='method')
+DatetimeProperties._add_delegate_accessors(
+    delegate=DatetimeIndex,
+    accessors=DatetimeIndex._datetimelike_ops,
+    typ='property')
+DatetimeProperties._add_delegate_accessors(
+    delegate=DatetimeIndex,
+    accessors=["to_period", "tz_localize", "tz_convert",
+               "normalize", "strftime", "round", "floor", "ceil"],
+    typ='method')
+
 
 class TimedeltaProperties(Properties):
     """
@@ -177,12 +198,15 @@ class TimedeltaProperties(Properties):
         """
         return self.values.components.set_index(self.index)
 
-TimedeltaProperties._add_delegate_accessors(delegate=TimedeltaIndex,
-                                            accessors=TimedeltaIndex._datetimelike_ops,
-                                            typ='property')
-TimedeltaProperties._add_delegate_accessors(delegate=TimedeltaIndex,
-                                            accessors=["to_pytimedelta", "total_seconds"],
-                                            typ='method')
+TimedeltaProperties._add_delegate_accessors(
+    delegate=TimedeltaIndex,
+    accessors=TimedeltaIndex._datetimelike_ops,
+    typ='property')
+TimedeltaProperties._add_delegate_accessors(
+    delegate=TimedeltaIndex,
+    accessors=["to_pytimedelta", "total_seconds", "round", "floor", "ceil"],
+    typ='method')
+
 
 class PeriodProperties(Properties):
     """
@@ -198,9 +222,10 @@ class PeriodProperties(Properties):
     Raises TypeError if the Series does not contain datetimelike values.
     """
 
-PeriodProperties._add_delegate_accessors(delegate=PeriodIndex,
-                                         accessors=PeriodIndex._datetimelike_ops,
-                                         typ='property')
+PeriodProperties._add_delegate_accessors(
+    delegate=PeriodIndex,
+    accessors=PeriodIndex._datetimelike_ops,
+    typ='property')
 PeriodProperties._add_delegate_accessors(delegate=PeriodIndex,
                                          accessors=["strftime"],
                                          typ='method')
@@ -215,8 +240,8 @@ class CombinedDatetimelikeProperties(DatetimeProperties, TimedeltaProperties):
 
 def _concat_compat(to_concat, axis=0):
     """
-    provide concatenation of an datetimelike array of arrays each of which is a single
-    M8[ns], datetimet64[ns, tz] or m8[ns] dtype
+    provide concatenation of an datetimelike array of arrays each of which is a
+    single M8[ns], datetimet64[ns, tz] or m8[ns] dtype
 
     Parameters
     ----------
@@ -230,14 +255,16 @@ def _concat_compat(to_concat, axis=0):
 
     def convert_to_pydatetime(x, axis):
         # coerce to an object dtype
-        if x.dtype == _NS_DTYPE:
 
-            if hasattr(x, 'tz'):
+        # if dtype is of datetimetz or timezone
+        if x.dtype.kind == _NS_DTYPE.kind:
+            if getattr(x, 'tz', None) is not None:
                 x = x.asobject
+            else:
+                shape = x.shape
+                x = tslib.ints_to_pydatetime(x.view(np.int64).ravel())
+                x = x.reshape(shape)
 
-            shape = x.shape
-            x = tslib.ints_to_pydatetime(x.view(np.int64).ravel())
-            x = x.reshape(shape)
         elif x.dtype == _TD_DTYPE:
             shape = x.shape
             x = tslib.ints_to_pytimedelta(x.view(np.int64).ravel())
@@ -250,20 +277,28 @@ def _concat_compat(to_concat, axis=0):
     # datetimetz
     if 'datetimetz' in typs:
 
+        # if to_concat have 'datetime' or 'object'
+        # then we need to coerce to object
+        if 'datetime' in typs or 'object' in typs:
+            to_concat = [convert_to_pydatetime(x, axis) for x in to_concat]
+            return np.concatenate(to_concat, axis=axis)
+
         # we require ALL of the same tz for datetimetz
-        tzs = set([ getattr(x,'tz',None) for x in to_concat ])-set([None])
+        tzs = set([getattr(x, 'tz', None) for x in to_concat]) - set([None])
         if len(tzs) == 1:
-            return DatetimeIndex(np.concatenate([ x.tz_localize(None).asi8 for x in to_concat ]), tz=list(tzs)[0])
+            return DatetimeIndex(np.concatenate([x.tz_localize(None).asi8
+                                                 for x in to_concat]),
+                                 tz=list(tzs)[0])
 
     # single dtype
     if len(typs) == 1:
 
-        if not len(typs-set(['datetime'])):
+        if not len(typs - set(['datetime'])):
             new_values = np.concatenate([x.view(np.int64) for x in to_concat],
                                         axis=axis)
             return new_values.view(_NS_DTYPE)
 
-        elif not len(typs-set(['timedelta'])):
+        elif not len(typs - set(['timedelta'])):
             new_values = np.concatenate([x.view(np.int64) for x in to_concat],
                                         axis=axis)
             return new_values.view(_TD_DTYPE)
@@ -271,4 +306,4 @@ def _concat_compat(to_concat, axis=0):
     # need to coerce to object
     to_concat = [convert_to_pydatetime(x, axis) for x in to_concat]
 
-    return np.concatenate(to_concat,axis=axis)
+    return np.concatenate(to_concat, axis=axis)
